export interface IComponent<T = any> {
    new (...args: any[]): T;
}
export declare type ComponentUpdater<T> = (value: T) => T | void;
export declare type ComponentUpdate<T> = [IComponent<T>, T];
export declare class Component<T> {
    value: T;
    constructor(value: T);
}
export declare class ID extends Component<string | number> {
}
export interface BaseEntity {
    ID: ID;
}
export declare type Entity<Components = any> = BaseEntity & {
    [componentName: string]: Components;
};
export declare type EntitiesCallback<Components> = (entities: Entity<Components>[]) => void;
export declare type ChangeCallback<Components> = (entity: Entity<Components>, updates: ComponentUpdate<Components>[]) => void;
interface Query<Components> {
    components: IComponent<Components>[];
    entities: Entity<Components>[];
    subscriptions: EntitiesCallback<Components>[];
    changeHandlers: ChangeCallback<Components>[];
}
export interface Config {
    parallel?: boolean;
    id?: () => string | number;
    onBefore?: (...args: any[]) => Promise<void>;
    onAfter?: (...args: any[]) => Promise<void>;
}
export declare function getID(entity: Entity): string | number;
export declare function getComponent<Component>(entity: Entity<Component>, component: IComponent<Component>): Component;
export declare function hasComponent<Component>(entity: Entity, components: IComponent<Component>): boolean;
export declare function hasComponents<Components>(entity: Entity, components: IComponent<Components>[]): boolean;
export declare function makeQueryKey(components: IComponent[]): string;
export declare class World {
    protected config: Config;
    protected systems: [Function, string][];
    protected entities: Map<string | number, Entity>;
    protected queries: {
        [key: string]: Query<unknown>;
    };
    constructor(config?: Config);
    protected queryWithKey<Components>(key: string, components: IComponent<Components>[], persist?: Boolean): Entity<Components>[];
    private _handleAddCallbacks;
    private _handleRemoveCallbacks;
    addComponent<Component>(entity: Entity<Component>, component: IComponent<Component>, ...args: any[]): void;
    addComponents<Components>(entity: Entity<Components>, components: [IComponent<Components>, ...any[]][]): void;
    createEntity<Components>(components: [IComponent<Components>, ...any[]][]): Entity<Components>;
    get<Components>(id: string): Entity<Components>;
    query<Components>(components: IComponent[], cache?: Boolean): Entity<Components>[];
    register(system: Function, components: IComponent[]): void;
    removeComponent(entity: Entity, component: IComponent): void;
    removeComponents(entity: Entity, components: IComponent[]): void;
    run<Args extends readonly any[]>(...args: Args): Promise<void>;
    subscribe<Components>(components: IComponent<Components>[], callback: EntitiesCallback<Components>, emit?: boolean): () => void;
    makeSubscription<Components>(components: IComponent<Components>[], emit?: boolean): (cb: EntitiesCallback<Components>) => () => void;
    handleChange<Components>(components: IComponent<Components>[], callback: ChangeCallback<Components>): () => void;
    makeChangeHandler<Components>(components: IComponent<Components>[]): (cb: ChangeCallback<Components>) => () => void;
    unsubscribe<Components>(components: IComponent[], callback: EntitiesCallback<Components>): void;
    deregisterHandler<Components>(components: IComponent[], callback: ChangeCallback<Components>): void;
    updateComponent<Component>(entity: Entity<Component>, Component: IComponent<Component>, update: Component | ComponentUpdater<Component>): Component;
    updateComponents<Components>(entity: Entity<Components>, updates: [
        IComponent<Components>,
        Components | ComponentUpdater<Components>
    ][]): [IComponent<Components>, Components][];
    private _update;
}
export {};
